@@ -3,9 +3,6 @@
 Module that exposes a command line interface for `matflow`.
 
 """
-
-from pathlib import Path
-
 import click
 
 from matflow import __version__
@@ -38,11 +35,12 @@ def go(workflow_path, directory=None):
 
 @cli.command()
 @click.option('--task-idx', '-t', type=click.INT, required=True)
+@click.option('--iteration-idx', '-i', type=click.INT, required=True)
 @click.option('--directory', '-d', type=click.Path(exists=True))
 @click.option('--array', is_flag=True)
-def prepare_task(task_idx, directory=None, array=False):
+def prepare_task(task_idx, iteration_idx, directory=None, array=False):
     print('matflow.cli.prepare_task', flush=True)
-    api.prepare_task(task_idx, directory, is_array=array)
+    api.prepare_task(task_idx, iteration_idx, directory, is_array=array)
 
 
 @cli.command()
@@ -57,11 +55,12 @@ def prepare_task_element(task_idx, element_idx, directory=None, array=False):
 
 @cli.command()
 @click.option('--task-idx', '-t', type=click.INT, required=True)
+@click.option('--iteration-idx', '-i', type=click.INT, required=True)
 @click.option('--directory', '-d', type=click.Path(exists=True))
 @click.option('--array', is_flag=True)
-def process_task(task_idx, directory=None, array=False):
+def process_task(task_idx, iteration_idx, directory=None, array=False):
     print('matflow.cli.process_task', flush=True)
-    api.process_task(task_idx, directory, is_array=array)
+    api.process_task(task_idx, iteration_idx, directory, is_array=array)
 
 
 @cli.command()
@@ -85,10 +84,11 @@ def run_python_task(task_idx, element_idx, directory=None):
 
 @cli.command()
 @click.option('--task-idx', '-t', type=click.INT, required=True)
+@click.option('--iteration-idx', '-i', type=click.INT, required=True)
 @click.option('--directory', '-d', type=click.Path(exists=True))
-def prepare_sources(task_idx, directory=None):
+def prepare_sources(task_idx, iteration_idx, directory=None):
     print('matflow.cli.prepare_sources', flush=True)
-    api.prepare_sources(task_idx, directory)
+    api.prepare_sources(task_idx, iteration_idx, directory)
 
 
 @cli.command()
@@ -105,7 +105,7 @@ def prepend_schema_source(schema_source_path):
 
 @cli.command()
 def validate():
-    'Load and validate task schemas against available extensions.'
+    """Load and validate task schemas against available extensions."""
     api.validate()
 
 
@@ -118,9 +118,23 @@ def cloud_connect(provider):
 @cli.command()
 @click.argument('directory', type=click.Path(exists=True))
 def kill(directory):
-    'Kill all pending and executing tasks.'
+    """Kill all pending and executing tasks."""
     api.kill(directory)
 
 
+@cli.command()
+@click.option('--iteration-idx', '-i', type=click.INT, required=True)
+@click.option('--directory', '-d', type=click.Path(exists=True))
+def write_element_directories(iteration_idx, directory=None):
+    api.write_element_directories(iteration_idx, directory)
+
+
+@cli.command()
+@click.argument('directory', type=click.Path(exists=True))
+@click.argument('archive')
+def archive(directory, archive):
+    api.archive(directory, archive)
+
+
 if __name__ == '__main__':
     cli()
