@@ -2,7 +2,6 @@ import os
 from pathlib import Path
 from warnings import warn
 
-from hpcflow.config import Config as hpcflow_config
 from ruamel.yaml import YAML, safe_load
 
 
@@ -16,10 +15,17 @@ class Config(object):
     __ALLOWED_CONFIG = [
         'task_schema_sources',
         'software_sources',
+        'default_run_options',
         'default_preparation_run_options',
         'default_processing_run_options',
+        'default_iterate_run_options',
+        'default_sticky_run_options',
+        'default_sticky_preparation_run_options',
+        'default_sticky_processing_run_options',
+        'default_sticky_iterate_run_options',
         'parallel_modes',
         'archive_locations',
+        'default_metadata',
     ]
 
     __conf = {}
@@ -103,25 +109,26 @@ class Config(object):
             raise ConfigurationError(msg)
 
         if 'software_sources' not in config_dat:
-            msg = (f'Missing `software_sources` from configuration file: {config_file}')
+            msg = f'Missing `software_sources` from configuration file: {config_file}'
             raise ConfigurationError(msg)
 
         return config_dat, config_file
 
     @staticmethod
-    def set_config(config_dir=None, raise_on_set=False):
-        'Load configuration from a YAML file.'
+    def set_config(config_dir=None, raise_on_set=False, refresh=False):
+        """Load configuration from a YAML file."""
 
         config_dir = Config.resolve_config_dir(config_dir)
 
         if Config._is_set:
             if raise_on_set:
                 raise ConfigurationError('Configuration is already set.')
-            return
+            elif not refresh:
+                return
 
         config_dat, _ = Config.get_config_file(config_dir)
-        schema_sources = [Path(i) for i in config_dat['task_schema_sources']]
-        software_sources = [Path(i) for i in config_dat['software_sources']]
+        schema_sources = [Path(i).expanduser() for i in config_dat['task_schema_sources']]
+        software_sources = [Path(i).expanduser() for i in config_dat['software_sources']]
 
         # Validate parallel_modes:
         ALLOWED_PARA_MODES = ['MPI', 'OpenMP']
@@ -222,7 +229,17 @@ class Config(object):
 
         Config.__conf['config_dir'] = config_dir
 
-        for i in ['default_preparation_run_options', 'default_processing_run_options']:
+        for i in [
+            'default_run_options',
+            'default_preparation_run_options',
+            'default_processing_run_options',
+            'default_iterate_run_options',
+            'default_sticky_run_options',
+            'default_sticky_preparation_run_options',
+            'default_sticky_processing_run_options',
+            'default_sticky_iterate_run_options',
+            'default_metadata',
+        ]:
             Config.__conf[i] = config_dat.get(i, {})
 
         hpcflow_config_dir = config_dir.joinpath('.hpcflow')
@@ -264,7 +281,7 @@ class Config(object):
 
     @staticmethod
     def _get_key_safe(key):
-        return (key[0], key[1], Config._get_software_safe(key[2]))
+        return key[0], key[1], Config._get_software_safe(key[2])
 
     @staticmethod
     def _validate_extension_setter():
@@ -284,7 +301,7 @@ class Config(object):
             if key not in Config.__conf['input_maps']:
                 Config.__conf['input_maps'].update({key: {}})
             if input_file in Config.__conf['input_maps'][key]:
-                msg = (f'Input file name "{input_file}" already exists in the input map.')
+                msg = f'Input file name "{input_file}" already exists in the input map.'
                 raise MatflowExtensionError(msg)
             Config.__conf['input_maps'][key][input_file] = func
 
@@ -295,7 +312,7 @@ class Config(object):
             if key not in Config.__conf['output_maps']:
                 Config.__conf['output_maps'].update({key: {}})
             if output_name in Config.__conf['output_maps'][key]:
-                msg = (f'Output name "{output_name}" already exists in the output map.')
+                msg = f'Output name "{output_name}" already exists in the output map.'
                 raise MatflowExtensionError(msg)
             Config.__conf['output_maps'][key][output_name] = func
 
@@ -304,7 +321,7 @@ class Config(object):
         if Config._validate_extension_setter():
             key = Config._get_key_safe(key)
             if key in Config.__conf['func_maps']:
-                msg = (f'Function map "{key}" already exists in the function map.')
+                msg = f'Function map "{key}" already exists in the function map.'
                 raise MatflowExtensionError(msg)
             Config.__conf['func_maps'][key] = func
 
@@ -349,7 +366,7 @@ class Config(object):
                 Config.__conf['output_file_maps'].update({key: {}})
             file_ref_full = '__file__' + file_reference
             if file_ref_full in Config.__conf['output_file_maps'][key]:
-                msg = (f'File name "{file_name}" already exists in the output files map.')
+                msg = f'File name "{file_name}" already exists in the output files map.'
                 raise MatflowExtensionError(msg)
             Config.__conf['output_file_maps'][key].update({file_ref_full: file_name})
 
