@@ -707,7 +707,7 @@ def validate_run_options(run_opts, type_label=''):
     if 'preparation' in type_label or 'processing' in type_label:
         ALLOWED += ['job_array']
 
-    bad_keys = set(run_opts.keys()) - set(ALLOWED)
+    bad_keys = set((run_opts or {}).keys()) - set(ALLOWED)
     if bad_keys:
         bad_keys_fmt = ', '.join([f'{i!r}' for i in bad_keys])
         raise TaskError(f'Run options not known: {bad_keys_fmt}.')
@@ -715,7 +715,7 @@ def validate_run_options(run_opts, type_label=''):
     run_opts = copy.deepcopy(run_opts)
 
     if 'preparation' in type_label:
-        if not run_opts:
+        if run_opts is None:
             run_opts = {
                 **Config.get('default_sticky_preparation_run_options'),
                 **Config.get('default_preparation_run_options'),
@@ -726,7 +726,7 @@ def validate_run_options(run_opts, type_label=''):
                 **run_opts
             }
     elif 'processing' in type_label:
-        if not run_opts:
+        if run_opts is None:
             run_opts = {
                 **Config.get('default_sticky_processing_run_options'),
                 **Config.get('default_processing_run_options'),
@@ -737,7 +737,7 @@ def validate_run_options(run_opts, type_label=''):
                 **run_opts
             }
     else:
-        if not run_opts:
+        if run_opts is None:
             run_opts = {
                 **Config.get('default_sticky_run_options'),
                 **Config.get('default_run_options'),
@@ -880,8 +880,14 @@ def validate_task_dict(task, is_from_file, all_software, all_task_schemas,
     task = {**def_keys, **copy.deepcopy(task)}
 
     all_run_opts = task.pop('run_options')
-    prep_run_opts = all_run_opts.pop('preparation', {})
-    proc_run_opts = all_run_opts.pop('processing', {})
+
+    if is_from_file:
+        prep_run_opts = task['prepare_run_options']
+        proc_run_opts = task['process_run_options']
+    else:
+        prep_run_opts = all_run_opts.pop('preparation', None)
+        proc_run_opts = all_run_opts.pop('processing', None)
+
     task['prepare_run_options'] = validate_run_options(prep_run_opts, '.preparation')
     task['process_run_options'] = validate_run_options(proc_run_opts, '.processing')
     task['run_options'] = validate_run_options(all_run_opts)
@@ -1100,7 +1106,6 @@ def get_input_groups(task_idx, task_lst, dependency_idx, element_idx,
 
         input_alias = non_local_input_i['alias']
         input_name = non_local_input_i['name']
-        input_context = non_local_input_i['context']
 
         group_name = task['schema'].get_input_by_alias(input_alias)['group']
         if group_name == 'default':
@@ -1122,7 +1127,7 @@ def get_input_groups(task_idx, task_lst, dependency_idx, element_idx,
             group_dict = element_idx[input_task_idx]['groups']
         else:
             # Parameter is imported from another workflow.
-            import_key = (input_name, input_context or DEFAULT_TASK_CONTEXT)
+            import_key = task_param_deps[input_alias]['from_import']
             group_dict = imported_parameters[import_key]['groups']
 
         group_names_fmt = ', '.join([f'"{i}"' for i in group_dict.keys()])
